<?php

if (!defined('_PS_VERSION_'))
    exit;

include_once (_PS_MODULE_DIR_ . 'flowpayment/lib-flow/flowlib.php');
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

class FlowPayment extends PaymentModule {

    protected $_errors = array();

    public function __construct() {
        $this->name = 'flowpayment';

        // Calling the parent's constructor. This must be done before any use of the $this->l() method, and after the creation of $this->name.
        parent::__construct();

        $this->displayName = $this->l('Flow Webpay');
        $this->description = $this->l('Pago con tarjetas de crédito o débito usando Flow');

        $this->author = 'Flow';
        $this->version = '1.0.0';
        $this->tab = 'payments_gateways';
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);

        // Module settings
        $this->setModuleSettings();

        // Check module requirements
        $this->checkModuleRequirements();
    }

    public function install() {
        if (!parent::install() OR !$this->registerHook('paymentOptions') OR !$this->registerHook('paymentReturn'))
            return false;
        return true;
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        $payment_options = [
            $this->getExternalPaymentOption()
        ];
        return $payment_options;
    }

    public function getExternalPaymentOption()
    {
        $recargo = $this->getRecargo();
        $params = $this->getFlowData();

        $externalOption = new PaymentOption();
        $externalOption->setCallToActionText("Flow - ".Configuration::get('FLOW_TITLE'))
            ->setAdditionalInformation($recargo)
            ->setAction($params["flow_action"])
            ->setInputs([
                'token' => [
                    'name' =>'parameters',
                    'type' =>'hidden',
                    'value' => $params["flow_request"]
                ],
            ])
            ->setLogo(Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . "modules/{$this->name}/logo-small.png");
        return $externalOption;
    }

    public function getContent() { 
        // Get active Shop ID for multistore shops
        $activeShopID = (int) Context::getContext()->shop->id;

        if (isset($_POST['flow_updateSettings'])) {
        	Configuration::updateValue('FLOW_TITLE', Tools::getValue('title'));        	
            Configuration::updateValue('FLOW_IDCOMERCIO', Tools::getValue('idComercio'));
            Configuration::updateValue('FLOW_PLATFORM'  , Tools::getValue('platformType'));
            Configuration::updateValue('FLOW_SKIPTYPE'  , Tools::getValue('skipType'));
            Configuration::updateValue('FLOW_ADDITIONAL'  , Tools::getValue('additional'));
            
            $hasFile = false;
            if (isset($_FILES['privateKey'])) {
            	$file = $_FILES['privateKey'];
            	$hasFile = $file['size']>0;
            	if ($hasFile) {
            		$fileName = $file['tmp_name'];
            		if (!$this->isValidPrivateKey($fileName)) {
            			$this->hasPrivateKey ='invalid';
            		} else {
            			Configuration::updateValue('FLOW_PRIVATEKEY', file_get_contents($fileName));
            			$this->hasPrivateKey ='valid';
            		}
            	}
            }
            
            $privateKey = Configuration::get('FLOW_PRIVATEKEY');
            
            if ($privateKey == null && !$hasFile) {
            	$this->hasPrivateKey = 'no';
            }
            Configuration::updateValue('FLOW_HASPRIVATEKEY', $this->hasPrivateKey);
            		
            $this->setModuleSettings();
            $this->checkModuleRequirements();

        } else {
        	$this->setModuleSettings();
        }
        
        $this->context->smarty->assign(array(
            'errors' => $this->_errors,
            'post_url' => Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']),
        	'data_platformType' => $this->platformType,
        	'data_idComercio' => $this->idComercio,
            'data_privateKeyInfo' => $this->hasPrivateKey=='valid'?'Tiene una clave privada válida registrada. Seleccione otra si la desea actualizar':'',
            'data_skipType' => $this->skipType,
        	'data_additional' => $this->additional,
        	'data_title' => $this->title,
        		
            'version' => $this->version,
            'img_header' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . "modules/{$this->name}/logo.png"
        ));

        return $this->display($this->name, 'views/templates/admin/config.tpl');
    }

    private function checkModuleRequirements() {
        $this->_errors = array();
        
        if ($this->title == '') {
        	$this->_errors['title'] = 'Debe indicar el nombre que se usará para este medio de pago';
        }
        if ($this->idComercio == '') {
        	$this->_errors['idComercio'] = 'Debe indicar su ID de Flow para recibir los pagos';
        }
        if ($this->additional == '') {
        	$this->_errors['additional'] = 'Si no tiene cobro adicional, indique 0';
        } else {
        	$additional = (float)$this->additional;
        	if ($additional<0 || $additional>100) {
        		$this->_errors['additional'] = 'Porcentaje debe estar entre 0 y 100';
        	}
        }
        
        if ($this->hasPrivateKey == 'invalid') {
        	$this->_errors['privateKey'] = 'El archivo no corresponde a una llave privada Flow';
        }
        if ($this->hasPrivateKey == 'no') {
        	$this->_errors['privateKey'] = 'Debe adjuntar su llave privada Flow (archivo .pem)';
        }
    }

    private function setModuleSettings() {
    	$this->title = Configuration::get('FLOW_TITLE');
        $this->idComercio = Configuration::get('FLOW_IDCOMERCIO');
        $this->hasPrivateKey = Configuration::get('FLOW_HASPRIVATEKEY');
        $this->platformType = Configuration::get('FLOW_PLATFORM');
        $this->skipType = Configuration::get('FLOW_SKIPTYPE');
        $this->additional = (float)Configuration::get('FLOW_ADDITIONAL');
    }
    
    private function isValidPrivateKey($privateKeyFile) {
    	$fp = fopen($privateKeyFile, "r");
    	$priv_key = fread($fp, 8192);
    	fclose($fp);
    	return openssl_get_privatekey($priv_key)!=null;
    }

    private function getRecargo()
    {
        $str = '';
        $cart = $this->context->cart;
        $recargo = (float)Configuration::get('FLOW_ADDITIONAL');

        if(!is_null($recargo) && is_numeric($recargo)  && $recargo > 0) {
            $monto_orden = (int)($cart->getOrderTotal(true, Cart::BOTH));
            $monto_adicional = round(($monto_orden * $recargo)/100.0);
            $str = 'Se aplicara un cargo adicional del '. $recargo . '%, equivalente a $ '.$monto_adicional.'';
        }

        return $str;
    }

    private function getFlowData() {
        $cart = $this->context->cart;
        $cartId = $cart->id;
        $customer = $this->context->customer;

        $comercio = Configuration::get('FLOW_IDCOMERCIO');
        $recargo = (float)Configuration::get('FLOW_ADDITIONAL');

        $ordenCompra = $cartId;
        $monto_orden = (int)($cart->getOrderTotal(true, Cart::BOTH));
        $monto_adicional = round(($monto_orden * $recargo)/100.0);
        $monto = $monto_orden + $monto_adicional;
        $monto_porcentaje = $recargo;

        $tipo_comision = Configuration::get('FLOW_PAYMENTYPE');
        $concepto = html_entity_decode('Orden #'.$ordenCompra.' de '.Configuration::get('PS_SHOP_NAME'), ENT_QUOTES, 'UTF-8');

        $id_producto = $customer->email."_".$cart->id;
        $key = Configuration::get('FLOW_PRIVATEKEY');

        $tipo_integracion = Configuration::get('FLOW_SKIPTYPE');
        $email = $customer->email;

        /* preparar pagina de exito o fracaso */
        $url_base = Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . "index.php?fc=module&module={$this->name}&controller=validate&cartId=" . $cartId;
        $url_exito   = $url_base."&return=ok";
        $url_fracaso = $url_base."&return=error";
        $url_confirmacion = Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . "modules/{$this->name}/validate.php";

        /*Compatibilidad*/
        $url_historia = $this->context->link->getPageLink('history');
        $url_exito = $url_historia;
        $url_fracaso = $url_historia;

        $flow_platform_id = Configuration::get('FLOW_PLATFORM');
        $isTestPlatform = !$flow_platform_id || $flow_platform_id == 'test';
        $flow_endpoint = $isTestPlatform?flow_get_endpoint_test():flow_get_endpoint();

        $data_action = $flow_endpoint;
        $medioPago = 1;
        $data_request = flow_pack($key, $comercio,
            $ordenCompra, $monto, $medioPago, $concepto, $id_producto,
            $url_exito, $url_fracaso, $url_confirmacion,
            $tipo_integracion, $email);

        $params = array(
            'flow_action' => $data_action,
            'flow_request' => $data_request
        );

        return $params;
    }
}

